##############################################################################
## FUNCTION NAME: yourgraph
##
## DESCRIPTION: This function generates summary graphs for a given gender and
## cause of disease.  It takes in as input either a file or an object
## generated by your.cast(). Alternatively, any data that is formatted in the
## manner described below can be used as the input. It then uses function
## plot.cast.new() to generate upto 2 graphs for every country.
##
##           graph 1: Time series of the data and the forecasts, one for each
##                    age group.
##
##           graph 2: Age profiles, both observed and predicted, one for each
##                    year.
##
## IMPORTED FUNCTIONS: 
##                     long.disease
##                     plot.cast
## INPUT:
##      input.data: Usually the output from your.cast(), which could either be
##                  an output file (e.g. output_allc_g2_modelOLS.dat) or an
##                  object (from a command such as yourcast.object <- your.cast())
##
##        yourcast: Logical variable telling the function whether or not your.cast()
##                  was used to generate the data.  (default: F)
##    dv.transform: a transformation to be performed on the data. (default: function(x){x})
##
## The next four parameters tells the program specifically which data to plot.
## The corresponding data is listed below:
##    obs.insample: observed data for the in-sample period (default:T)
##   pred.insample: predicted data for the in-sample period (default:T)
##   obs.outsample: observed data for the out-sample period (default:T)
##  pred.outsample: observed data for the out-sample period (default:T)
##
##The remaining are all parameters relating to the layout of the graphs.
##
##plot.time.series: Logical variable telling the function whether or
##                  not to plot the time series graph. (default: T)
##
##plot.age.profiles: Logical variable telling the function whether or
##                   not to plot the age profiles graph. (default: T)
##
##            wide: width of the graphics region in the postscript file. 
##                  (default: 8)
##
##            high: height of the graphics region in the postscript file. 
##                  (default is 8)
##
## page.horizontal: logical input specifying orientation of the image in the postscript file.
##                  (default:F)
##
##         row.num: number of rows into which graphs should be organized. (default: 1)
##
##         col.num: number of columns into which graphs should be organized. (default: 2)
##
##  background.col: background color for the postscript file. (default: "tan1")
##
##      print2file: logical input specifying whether output should be printed to
##                  an .eps file or to the screen. (default:.eps file)
##
##
##  NOTE: the user can provide her own mortality data for a given cause and
##  gender in the format of a file or a list.  The following elements should be
##  present in the file or list.
##
##  plotting .data: a list where item is a M x N matrix of log-mortality
##                  rates for a given country, where M and N correspond
##                  with years and age groups respectively.  The name of
##                  each item should be the country name as it should appear
##                  in the labels of the graphs.
##
##        depvar : a character string specifying the cause of death
##
##       strata   : "m" for male and "f" for female
##     cntry.lst : a vector that stores the country names
##
##     dispgraph: character for command to pipe the graphics into operating system
##                if null, then graphics are saved but no dispaly
##
## WRITTEN BY:  Federico Girosi and Nirmala Ravishankar
##              CBRSS, Harvard University
##
## last revised : Thursday, September 22nd, 2004
##
##############################################################################
yourgraph <- function(input.data, yourcast = F, 
                      obs.insample= T, obs.outsample = T,
                      pred.insample = T, pred.outsample = T,
                      wide = 8, high = 8, plot.time.series = T,
                      plot.age.profiles = T, page.horizontal = FALSE,
                      row.num = 1, col.num = 2, background.col = "tan1",
                      print2file = T, dv.transform = function(x){x},
                      depvartext = long.causes, strata=NULL,
                      dirpath=NULL, ylim=NA, dispgraph="acroread"){
               
  
  ##Creating directory path name
  ev <- environment()
  if(length(dirpath) <= 0)
    dirpath <- paste(getwd(), "/",sep="")
  else
    dirpath <- paste(dirpath,"/", sep="")
 
  ## Next we load the input file, which is usually the output from your.cast().
  ## It can either be a data file or an object. If it is not generated
  ## from your.cast(), then it should be configured as per the specifications
  ## of the yourcast object.
  
  if (is.character(input.data)){
    cat("Reading in data file...", "\n")   
    file <- paste(dirpath, input.data, sep="")
    input.data <- load(file, envir=ev)
    if(is.symbol(input.data))
      input.data <- eval(as.symbol(input.data))
  } else 
  cat("Obtaining data object...", "\n")
  
  data.vec <-  c("cntry.lst", "depvar", "strata", "insampy", "outsampy",
                 "yhatout", "yhatin", "model")
   
  input.data <- convert.input.graph(input.data, strata)

  cntry.lst <- input.data$cntry.lst
  depvar <- input.data$depvar
  model <- input.data$model
  strata <- input.data$strata
  insampy <- input.data$insampy
  outsampy <- input.data$outsampy
  yhatin <- input.data$yhatin
  yhatout <- input.data$yhatout
  
  data.sym <- sapply(data.vec,as.symbol)
  
  check <- sapply(data.sym, eval, env=ev)
  
  insampy <- lapply(insampy,FUN = dv.transform) 
  outsampy <- lapply(outsampy,FUN = dv.transform)
  yhatin <- lapply(yhatin,FUN = dv.transform)
  yhatout <- lapply(lapply(yhatout, na.omit), FUN = dv.transform)
 
  
 ## Creating disease, gender and country name strings for labels
  disease.name <- NULL
  depvarN <- extract.depvar.graph(depvar)$numerator
  
  if(is.function(depvartext)){
    lname <- depvartext(depvar)
 
    if(!identical(lname,depvar) ){
      lname <- depvartext(depvarN)
      disease.name <- ifelse( is.list(lname), lname[[1]], lname[1])
   
    }
  }
  if(length(disease.name) <= 0)
    disease.name <- depvar
  
  nm <- names(cntry.lst)
  cntry.lst <- as.data.frame(cntry.lst)
  if(is.data.frame(cntry.lst))
    ln <- length(cntry.lst)
  
  
  
  if(ln >= 2){
    countries <- as.matrix(cntry.lst[1])
    c.name <- as.matrix(cntry.lst[2])
  }else{
    countries <- as.matrix(cntry.lst)
    c.name <- countries
  }
  if (length(strata) > 0 && is.numeric(strata)){
    if (strata==2) {
      gender.str <- "m"}
    else if (strata==3){
      gender.str <- "f"}
    else
      gender.str <- as.character(strata)
  } else if (length(strata) > 0)
     gender.str <- strata
  else
    gender.str <-  NULL
  


  ##The following does the actual plotting using plot.cast.new()
  if (sum(pred.insample == T, pred.outsample == T,
          obs.insample == T, obs.outsample ==T) == 0){
    ## The above checks if there is any data to plot.
    cat("No data found for plotting.", "\n")
    
 } else {
    if (print2file == T){
     
     
     
      if(length(strata) > 0 )
        outfilename <- paste("trends_", depvarN, "_", gender.str,
                             "_model_", model, ".pdf", sep = "")
      else
        outfilename <- paste("trends_", depvarN, "_", 
                             "model_", model, ".pdf", sep = "")
   
      outfile <- paste(dirpath, outfilename, sep = "")
      ## Creating the output file
      cat("The file ",outfile, " is being written into disk...","\n")
      
      pdf(outfile,pointsize = 14,width=wide,family="Helvetica",
          bg =background.col, height=high)}
    
    par(mfrow = c(row.num,col.num))
    par(cex.main = 1, cex.lab = 0.75, cex.axis = 0.5)
    par(mar = c(4, 4, 2, 1)) 
   
    ## The following code extracts the data for each country, formats it and then
    ## uses plot.new.graph() to plot the selected graphs.
    
    for (i in 1:length(countries)){
      
      pred.in <- yhatin[[i]]
      pred.out <- yhatout[[i]]
      obs.in <- insampy[[i]]
      obs.out <- outsampy[[i]]
      
      ##
      cntry.str <- c.name[i]
      
      ## Setting graphing parameters like background color, layout of the page 
     
     ### par(bg= background.col);
  
      try(plotcast.new(pred.in, pred.out, obs.in, obs.out, disease.name, gender.str,
                       cntry.str, print2file,plot.time.series,pred.outsample,
                       pred.insample,obs.outsample,obs.insample, plot.age.profiles, ylim=ylim), silent=T)
     
    }
    dev.off()
  }
  if(is.character(dispgraph))
    res <- try(readLines(pipe(paste(dispgraph, outfile))), silent=T)
}

### DESCRIPTION: Takes the list output of yourcast and converts it
###              to the parameters and inputs arguments taken with yourgraph.
###
### INPUT: indata the object output of function yourcast;
###        gender is optional and use to label the graphics
###
### OUTPUT: list with elements, strata, model, country list (codes & names)
###         insample and outsample lists for dependent variable y, and prediction yhat
###
### AUTHOR Elena villalon
###        IQSS Harvard Univ
###        evillalon@iq.harvard.edu
###        02/23/2006
##########################################################
      
  convert.input.graph <- function(indata, gender){
    ev <- environment()
    loadone <- FALSE
    output <- list()
    output <- c(output, strata=gender)
    call <- indata$call

    model <- call$model
       
    output <- c(output, model= list(model))
    ff <- na.omit(call$formula)
    if(length(ff) <= 0)
      {
        load("yourcast.savetmp", envir=ev)
        loadone <- TRUE
        ff <- formula
      }
   
 
      ###function to get the depvar or lhs of formula

    ffc <- as.character(ff)
    depvar <- trim.blanks(ffc[2])

   
###
    output <- c(output, depvar=list(depvar))
    dobj0  <- na.omit(call$dataobj)
    if(length(dobj0) <= 0 && loadone)
      dobj <- eval(as.symbol(dobj))
    else if(length(dobj0) <= 0){
      load("yourcast.savetmp", envir=ev)
      loadone <- TRUE
      dobj <- eval(as.symbol(dobj))
    }else
    dobj <- dobj0
      
      
    if(is.character(dobj))
      dobj <- eval(as.symbol(dobj))
   
    icode <- dobj$index.code
   
    if(length(icode) > 0){
      spicode <- strsplit(icode, NULL)[[1]]
      ind <- grep("[g G]", spicode)
      nogs <- length(ind)
      fg <- ifelse(!is.element(1, ind), min(ind), 1)
      lg <- fg + nogs - 1
    }
    csid  <- names(dobj$data)
   
    cntry.codes <- sapply(csid, function(codx, fg, lg) {
      tosplit <- strsplit(codx, NULL)[[1]]
      ret <- paste(tosplit[fg:lg], collapse="")
      return(ret)},fg, lg)
 
    cntry.codes <- sapply(cntry.codes, FUN=trim.blanks)
    cntry.codes <- unique.default(unlist(cntry.codes))
###    print(cntry.codes)
### function for cntry names
    G.names <- dobj$G.names
    cntry.lst <- countries(cntry.codes, G.names)
###    print(cntry.lst)
###
    output <- c(output, cntry.lst=list(cntry.lst))
    datamat <- dobj$data
    yhat <- indata$yhat
   
### function for components of y insample, outsample and smoothing
    frm <- call$sample.frame
     
    yy <- getally(yhat,frm)
     
      ###
    insampy  <- yy$insampy
    outsampy <- yy$outsampy
    yhatin   <- yy$yhatin
    yhatout <- yy$yhatout
    whodigits <- findigits.graph(icode)
     

    who.digit.first <- whodigits$who.digit.first
      
    who.cntry.digits <- whodigits$who.cntry.digits
    who.age.digits <- whodigits$who.age.digits
    who.year.digits <- whodigits$who.year.digits
       
      
    insampy <- list.by.cntry(x=insampy,ebase=NULL,who.digit.first=who.digit.first,
                             who.cntry.digits=who.cntry.digits,who.age.digits=who.age.digits,
                             who.year.digits=who.year.digits)
    
    outsampy <- list.by.cntry(x=outsampy,ebase=NULL,who.digit.first=who.digit.first,
                              who.cntry.digits=who.cntry.digits,who.age.digits=who.age.digits,
                              who.year.digits=who.year.digits)
    yhatin <- list.by.cntry(x=yhatin,ebase=NULL,who.digit.first=who.digit.first,
                            who.cntry.digits=who.cntry.digits,who.age.digits=who.age.digits,
                            who.year.digits=who.year.digits)
    yhatout <- list.by.cntry(x=yhatout,ebase=NULL,who.digit.first=who.digit.first,
                               who.cntry.digits=who.cntry.digits,who.age.digits=who.age.digits,
                               who.year.digits=who.year.digits)
      
    output <- c(output, insampy = list(insampy), outsampy=list(outsampy),
                yhatin=list(yhatin), yhatout=list(yhatout))
    return(output)
  }

### DESCRIPTION: Takes the yhat element of the list output of yourcast and 
###              obtain insample and outsample components for the depvar data and
###              model estimate yhat of the dependent variable. 
###
### INPUT: yhat the list output of function yourcast with the dependent variable and yhat;
###        frm the sample frame with the insample and outsample years limits. 
###
### OUTPUT: insample and outsample lists for dependent variable and the model predictions; 
###         insampy, outsampy (depenedent variable data),
###         yhatin and yhatout the model estimates for dependent variable
###
### AUTHOR Elena Villalon
###        IQSS Harvard Univ
###        evillalon@iq.harvard.edu
###        02/23/2006
##########################################################
 getally <- function(yhat,frm)
  {
     
     frm <- as.character(frm)
     ix <- grep("c", frm)
     if(length(ix) > 0)
       frm <- frm[-ix]
     frm <- as.numeric(frm)
     
     partion <- lapply(yhat, function(mat,frm)
                split.data.frame(mat, as.numeric(rownames(mat)) <= frm[2]), frm)
     inpart  <-  lapply(partion, function(mat) mat$T)
     outpart <- lapply(partion, function(mat) mat$F)
     insampy <- lapply(inpart, function(mat) as.matrix(as.matrix(mat)[,1]))
     outsampy <- lapply(outpart, function(mat) as.matrix(as.matrix(mat)[,1]))
     yhatin <- lapply(inpart, function(mat) as.matrix(as.matrix(mat)[,2]))
     yhatout <- lapply(outpart, function(mat) as.matrix(as.matrix(mat)[,2]))
    
    
     lst <- list(insampy=insampy, outsampy=outsampy, yhatin=yhatin, yhatout=yhatout)
   
     return(lst)                  
  }
### DESCRIPTION: Takes the formula and extract the dependent variable or left-hand-side
###
### INPUT: a formula such as log(allc/pop) ~ gdp^3 + tobacco + time
###
### OUTPUT: left-hand-side of formula with its components, e.g. depvarN=allc, depvarD= pop; 
###
### AUTHOR Elena Villalon
###        IQSS Harvard Univ
###        evillalon@iq.harvard.edu
###        02/23/2006
#######################################################################
 extract.depvar.graph <- function(ff)
  {
    if(!is.character(ff)){
      ffc <- as.character(ff)
      lhs <- ffc[2]
  }else
    lhs <- ff
    
    depvf <- findregex(lhs)
  
    depvar <- split.ops(depvf)
    
    if(length(depvar) > 1){
      depvarN <- depvar[1]
      depvarD <- depvar[2]
    }else{
       depvarN <- depvar
       depvarD <- NULL
     }
    lst <- c(list(numerator=depvarN), list(denominator=depvarD))
  return(lst)    
  }

### DESCRIPTION: Takes the codes for countries in yourcast sim and finds the names.
###
### INPUT: a vector codes with country codes;
###        gnames matrix or data.frame with names associated to codes
###
### OUTPUT: the matrix or data.frame with names and codes for the specific codes 
###
### AUTHOR Elena Villalon
###        IQSS Harvard Univ
###        evillalon@iq.harvard.edu
###        02/23/2006
#######################################################################

countries <- function(codes, gnames){
  if(length(gnames) <= 0)
    return(codes)
  
  bb <- is.matrix(gnames) || is.data.frame(gnames)
  if(!bb)
      gnames <- matrix(gnames, ncol=2, byrow=T)
  
    allcodes <- gnames[, 1]
    codes <- as.numeric(codes)
    ind <- sapply(codes, match,allcodes)
    return(gnames[ind,])   
    
}

                      
##**************************************************************************
##**************************************************************************

### for display in the graphs
### Elena Villalon: Because with the revised version of yourcast the data part of dataobj contains
### columns such as allc2, allc3, cvds2, cvds3 for male (2) and females we need to change
### the input x so that it recognizes the disease
###
long.causes <- function(x,dispgender=T){
  nc <- nchar(x)
  xs <- strsplit(x, NULL)[[1]]
  ix2 <- grep("2", xs)
  ix3 <- grep("3", xs)
  ix <- c(ix2, ix3)
  str <- NULL
  if(length(ix2) > 0)
    str <- "m"
  if(length(ix3) > 0)
    str <- "f"
  
  if(length(ix) > 0){
    xs <- xs[-ix]
    x <- paste(xs, collapse="")
  }
  disnam1 <- switch(paste(x),
                    allc =    "All",
                    malr =    "Malaria", 
                    aids =    "AIDS", 
                    tubr =    "Tuberculosis", 
                    otin =    "Other",
                    lung =    "Lung", 
                    molp =    "Cancer of",
                    livc =    "Liver", 
                    stom =    "Stomach",
                    brst =    "Breast",
                    cerv =    "Cervix",
                    omal =    "Other",
                    rspi =    "Respiratory",
                    rspc =    "Respiratory",
                    cvds =    "Cardiovascular",
                    dgst =    "Digestive",
                    matc =    "Maternal",
                    pern =    "Perinatal",
                    allo =    "All",
                    trns =    "Transportation",
                    unin =    "Other",
                    suic =    "Suicide", 
                    homi =    "Homicide",   
                    ward =    "War");

  disnam2 <- switch(paste(x),
                    allc =    "Causes", 
                    malr =    "", 
                    aids =    "", 
                    tubr =    "", 
                    otin =    "Infectious",
                    lung =    "Cancer", 
                    molp =    "Mouth and",
                    livc =    "Cancer", 
                    stom =    "Cancer", 
                    brst =    "Cancer", 
                    cerv =    "Cancer", 
                    omal =    "Malignant", 
                    rspi =    "Disease,", 
                    rspc =    "Disease,", 
                    cvds =    "Disease", 
                    dgst =    "Disease", 
                    matc =    "Conditions",
                    pern =    "Conditions", 
                    allo =    "Other", 
                    trns =    "Accidents", 
                    unin =    "Unintentional", 
                    suic =    "", 
                    homi =    "",   
                    ward =    "");

  disnam3 <- switch(paste(x),
                    allc =    "", 
                    malr =    "", 
                    aids =    "", 
                    tubr =    "", 
                    otin =    "Diseases", 
                    lung =    "", 
                    molp =    "Esophagus", 
                    livc =    "", 
                    stom =    "", 
                    brst =    "",
                    cerv =    "",
                    omal =    "Neoplasms", 
                    rspi =    "Infectious", 
                    rspc =    "Chronic", 
                    cvds =    "",
                    dgst =    "",
                    matc =    "",
                    pern =    "",
                    allo =    "Diseases", 
                    trns =    "",
                    unin =    "Injuries", 
                    suic =    "",
                    homi =    "",
                    ward =    "");
  
  name <- paste(disnam1,disnam2,disnam3);
  if(length(str) >0 && dispgender)
    name <- paste(name,"(",str,")", sep="")
  
  return(list(name,disnam1,disnam2,disnam3));
}
### DESCRIPTION: Given the index.code finds the number of digits for country, age, year.
###
### INPUT: a scalar character string with index code,e.g icode="__ggggaatttt"
###
### OUTPUT: a list with digits numbers, who.digit.first=2,
###         who.cntry.digits=4, who.age.digits = 2, who.year.digits=4
###
### AUTHOR Elena Villalon
###        IQSS Harvard Univ
###        evillalon@iq.harvard.edu
###        02/23/2006
#######################################################################

 findigits.graph <- function(icode=NULL)
  {
    lst <- list()
    
    if(length(icode) <= 0)
      return(lst)
       
    spicode <- strsplit(icode, NULL)[[1]]
    ind <- grep("[g G]", spicode)
    nogs <- length(ind)
    fg <- ifelse(!is.element(1, ind), min(ind), 1)
    who.digit.first <- ifelse(fg <= 1, 0, fg-1)
    lst <- c(lst, who.digit.first=who.digit.first)
    who.cntry.digits <- nogs
    lst <- c(lst,  who.cntry.digits= who.cntry.digits)
    ind <- grep("[a A]", spicode)
    noas <- length(ind)
    who.age.digits <- noas
    lst <- c(lst, who.age.digits= who.age.digits)
    ind <- grep("[t T]", spicode)
    nots <- length(ind)
    who.year.digits <- nots
    lst <- c(lst, who.year.digits= who.year.digits)
    
    return(lst)
    
  }
